import os, platform, subprocess, sys, re
from os.path import exists

def shOut(cmd):
    """Return stdout if command was successful, otherwise raise an Exception."""
    print(f"Debug: in shOut({repr(cmd)})")
    if '|' in cmd: raise ValueError("Piping is forbidden")
    try:
        result = subprocess.run(
            # To be compatible with Python 3.6, we need to avoid arguments `text`,
            # `capture_output` and `encoding`.
            cmd,
            shell=(type(cmd)==str),
            check=True,
            universal_newlines=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)
    except subprocess.CalledProcessError as e:
        # Accessing stdout and stderr from the exception object
        print("COMMAND:")
        print(cmd)
        print("STDOUT:")
        print(e.stdout)
        print("STDERR:")
        print(e.stderr)
        raise e
    return result.stdout.strip()

def stdin_for_process_hierarchy(pid):
    print(f"Debug: in stdin_for_process_hierarchy({repr(pid)})")
    if not isinstance(pid, int) or pid <= 1:
        return []
    # Find stdin
    stdin_path=None
    try:
        open_files=shOut(f"lsof -p {pid} -Ffn")
    except subprocess.CalledProcessError as e:
        print(e)
        return []
    lines = open_files.splitlines()
    for i, line in enumerate(lines):
        if line == "f0" and i + 1 < len(lines) and lines[i + 1].startswith("n"):
            stdin_path=lines[i + 1][1:]
            break
    ppid=int(shOut(f"ps -o ppid= -p {pid}") or "0")
    print(f"Debug stdin_path={repr(stdin_path)}")
    if stdin_path and exists(stdin_path):
        print(shOut(["ls", "-la", stdin_path]))
        stdin_path=shOut(["readlink", "-f", stdin_path])
    else:
        stdin_path=None
    if stdin_path:
        return [stdin_path, *stdin_for_process_hierarchy(ppid)]
    return stdin_for_process_hierarchy(ppid)

def try_interactive_setup_with_parent_tty():
    print("Debug: in try_interactive_setup_with_parent_tty()")
    for parent_stdin_path in stdin_for_process_hierarchy(os.getpid()):
        if not parent_stdin_path or not (
                parent_stdin_path.startswith("/dev/tty") or
                parent_stdin_path.startswith("/dev/pts/")):
            continue
        with open(parent_stdin_path, 'r') as effective_stdin:
            if not os.isatty(effective_stdin.fileno()):
                continue
            orig_stdin = sys.stdin
            try:
                sys.stdin = effective_stdin
                return interactive_setup_help_pr()
            finally:
                sys.stdin = orig_stdin
    print("Detected piped input but could not find a terminal, which is necessary for")
    print("interactive questions.")

def interactive_setup_help_pr():
    print("Debug: in interactive_setup_help_pr()")
    def ask(question, validate=None):
        while True:
            user_input = input(question + " ")
            print()
            if validate is None or validate(user_input):
                return user_input
            else:
                print("Invalid input, please try again.")
    def askYN(question):
        answer = ask(question + " [y/n]", lambda x: x.strip().lower() in ["yes", "y", "no", "n"])
        return answer.strip().lower() in ["yes", "y"]
    if(askYN("Are you lucky?")):
        print("Good for you! 👍")
    else:
        print("Sorry to hear that... here, have a cookie! 🥠")

def main():
    if os.isatty(1):
        if os.isatty(0):
            interactive_setup_help_pr()
        else:
            # stdin is not a tty even though stdout is. This indicates
            # piping, which is used in the one-line setup.
            try_interactive_setup_with_parent_tty()
    else:
        print("Cannot use terminal for interactive questions.")

main()

# curl -fLsS https://repo.hops.works/dev/axel.svensson/tt.py | python3
# curl -fLsS https://repo.hops.works/dev/axel.svensson/tt.py | sudo python3

# bash -c 'set -o pipefail; curl -fLsS https://repo.hops.works/dev/axel.svensson/tt.py | python3'
# bash -c 'set -o pipefail; curl -fLsS https://repo.hops.works/dev/axel.svensson/tt.py | sudo python3'

# todo: run next to a file named 'pipe'.
